@@ -17,7 +17,7 @@ sub new {
     my @extra_compiler_flags = (qw/
         -Wall -Wextra -Wno-parentheses
         -Wno-unused -Wno-unused-parameter
-    /, ('-I.', "-I$LIBZSTD_DIR"));
+    /, ('-I.', "-I$LIBZSTD_DIR"), ('-DZSTD_LEGACY_MULTITHREADED_API'));
     my $ld = $self->config('ld');
     if ($ld =~ s/^\s*env MACOSX_DEPLOYMENT_TARGET=[^\s]+ //) {
         $self->config(ld => $ld);
@@ -26,7 +26,7 @@ sub new {
         $self->config(optimize => '-g -O0');
     }
     $self->extra_compiler_flags(@extra_compiler_flags);
-    $self->extra_linker_flags("-L$LIBZSTD_DIR", "-lzstd");
+    $self->extra_linker_flags('-pthread', "$LIBZSTD_DIR/libzstd.a");
     $self;
 }
 
@@ -38,7 +38,7 @@ sub ACTION_build {
     my $self = shift;
     $self->ACTION_ppport_h() unless -e 'ppport.h';
     unless (-f "$LIBZSTD_DIR/libzstd.a") {
-        local $ENV{CFLAGS} = '-O3 -fPIC';
+        local $ENV{CFLAGS} = '-DZSTD_MULTITHREAD -DZSTD_LEGACY_MULTITHREADED_API -O3 -fPIC';
         my $make = 'make';
         $make = 'gmake' if $^O =~ /bsd$/ && $^O !~ /gnukfreebsd$/;
         $self->do_system($make => '-C', $LIBZSTD_DIR, 'libzstd.a');
